from .tags_mandatory import FaoTagsStrategy
from .tags_scheduler import TagScheduler


class Tags:
    def __init__(self, **kwargs):
        self.mandatory = FaoTagsStrategy(**kwargs)
        self.scheduler = TagScheduler(**kwargs)



from pydantic import Field

from pydantic_settings import BaseSettings, SettingsConfigDict

class DefaultSettings(BaseSettings):
    model_config = SettingsConfigDict(extra='ignore')
    environment: str
    app_name: str = Field(..., alias="application_name")


from typing import Generic, TypeVar, Optional
from aws_cdk import Environment
from os import getenv, path

from .environment_parameters import BaseAccount, EnvironmentParameters
from .default_settings import DefaultSettings
from .tags import Tags

TAccount = TypeVar('TAccount', bound=BaseAccount)  # EnvironmentParameters extension if we update python to 3.12 we can use DefaultAccount as default
TSettings = TypeVar('TSettings', bound=DefaultSettings)    # Settings model

class ConfigProvider(Generic[TSettings, TAccount]):
    """ This class aggregates all config passed to the CDK App. Config comes from the following sources:
        - environments_parameters, provided by the cloud team
        - settings, provided in the pipeline for each environment
    """
    _instance: Optional["ConfigProvider[TSettings, TAccount]"] = None

    def __new__(cls, environments_parameters: EnvironmentParameters[TAccount], settings: TSettings, relative_path: str = '../'):
        if cls._instance is None:
            cls._instance = super(ConfigProvider, cls).__new__(cls)
            cls._instance.__init_once(environments_parameters, settings, relative_path)
        return cls._instance

    def __init_once(self, environments_parameters: EnvironmentParameters[TAccount], settings: TSettings, relative_path: str):

        if hasattr(self, '_initialized') and self._initialized:
            return

        self.env, self.env_lower = self._get_env()
        self._venv_files = self._get_venv_files(relative_path)
        self._venv_files_encoding = 'utf-8'
        self._check_venv_files_exists()

        self.envs_params = environments_parameters.model_dump()
        self.env_params: TAccount = getattr(environments_parameters.accounts, self.env_lower)  # Fetching the current environment
        self.tags = Tags(_env_file=self._venv_files, _env_file_encoding=self._venv_files_encoding)
        self.cdk_env = Environment(account=self.env_params.id, region='eu-west-1')

        self.settings: TSettings = settings(_env_file=self._venv_files,
                                 _env_file_encoding=self._venv_files_encoding)  # Directly storing settings instance
        
        self.app_name = self.settings.app_name  # Accessing the app name from settings

        self._initialized = True

    @staticmethod
    def _get_env() -> (str, str):
        env = getenv("ENVIRONMENT", "Development")
        return env, env.lower()

    def _get_venv_files(self, relative_path: str) -> list[str]:
        dotenv_file = f'{relative_path}/.env.{self.env_lower}-iac'
        dotenv_file_local = f'{relative_path}/.env.{self.env_lower}-iac.private'

        env_files = []
        if path.isfile(dotenv_file):
            print(f'Using {dotenv_file}')
            env_files.append(dotenv_file)
        else:
            print(f'Not found {dotenv_file}')

        if not getenv("CI"):
            print(f'Local run. Trying to fetch private env file {dotenv_file_local}')
            if path.isfile(dotenv_file_local):
                env_files.append(dotenv_file_local)
            else:
                print(f'Not found {dotenv_file_local}')

        return env_files

    def _check_venv_files_exists(self) -> None:
        for file in self._venv_files:
            if not path.isfile(file):
                raise Exception(f'Expected venv file {file} does not exist')

    # app is not typed to avoid depending on aws cdk for this package
    def apply_tags(self, app):
        self.tags.mandatory.apply_tags(application=app, tag_list=self.tags.mandatory.model_dump())



from pydantic_settings import BaseSettings, SettingsConfigDict


class TagScheduler(BaseSettings):
    """
    Class to aggregate all Scheduler configuration tags
    """
    model_config = SettingsConfigDict(extra='ignore', env_prefix='tag_scheduler_')
    uptime: str = "08:00-18:00"
    uptime_days: str = "1-2-3-4-5"
    uptime_skip: str = "false"



from typing import List, Optional, TypeVar, Generic, Any, Dict
from pydantic import BaseModel

# Define a type variable for generic environment-specific account settings
G = TypeVar('G', bound=BaseModel)

# Base model for AccountsDict to rigidly define the structure of accounts
class AccountsDict(BaseModel, Generic[G]):
    development: G
    qa: G
    production: G
    sharedservices: Optional[Any]
    sandbox: Optional[Any]

# Base model for EnvironmentParameters using Pydantic
class EnvironmentParameters(BaseModel, Generic[G]):
    accounts: AccountsDict[G]
    networking: Optional[Any]  # You can replace Any with a specific model for networking if desired

class BaseAccount(BaseModel):
    name: str
    id: str

class DefaultAccount(BaseAccount):
    name: str
    id: str
    vpc: str
    public_subnet_ids: List[str]
    private_subnet_ids: List[str]
    availability_zones: List[str]
    az: str
    bastion_host_security_group: str
    bastion_host_production_control_security_group: str
    bastion_host_windows_admin_security_group: str
    scan_target_security_group: str
    asg_sns_topic: str
    asg_cw_alerts_lc_hooks_role: str
    asg_cw_alerts_lc_hooks_launch_sns: str
    asg_cw_alerts_lc_hooks_terminate_sns: str
    web_acl: str
    s3_config_bucket: str
    smtp_relay_security_group: str
    smtp_server_endpoint: str
    smtp_server_port: str
    kms_ssm_key: str
    kms_ebs_key: str
    kms_rds_key: str
    kms_vault_key: str
    kms_neptune_key: str
    app_proxy_security_group: str
    cognito_user_pool_arn: str
    cognito_domain: str
    ssl_certificate_star_fao_org_arn: str
    ssl_certificate_star_fao_org_arn_north_virginia: str
    oracle_oem_client_security_group: str
    cloudfront_only_access_security_group: str
    ldap_access_security_group: str
    domain_control_access_security_group: str
    route53_hosted_zone_id: Optional[str] = None
    route53_domain_name: Optional[str] = None


from typing import Dict

from aws_cdk import Tags, App
from pydantic import Field

from pydantic_settings import BaseSettings, SettingsConfigDict


class FaoTagsStrategy(BaseSettings):
    """
    All mandatory tags by the Cloud Team
    """
    model_config = SettingsConfigDict(extra='ignore')
    ApplicationName: str = Field(..., alias="application_name")
    Environment: str = Field(..., alias="environment")
    BusinessOwner1: str = Field(default=None, alias="business_owner")
    BudgetCode1: str = Field(default=None, alias="budget_code")
    BudgetHolder1: str = Field(default=None, alias="budget_holder")

    @staticmethod
    def apply_tags(application: App, tag_list: Dict) -> None:
        for tag, value in tag_list.items():
            if value:
                Tags.of(application).add(
                    tag,
                    value,
                    apply_to_launched_instances=True,
                )
